@@ -57,22 +57,22 @@
 #define DELAY_MAX_TICKS 64
 #define INITIAL_TICKS   2
 
-#define EPHY_MIME_TYPE_PERMISSIONS_URI "/org/gnome/epiphany/mime-types-permissions.xml"
-
 typedef enum {
   EPHY_PROFILE_DIR_UNKNOWN,
   EPHY_PROFILE_DIR_DEFAULT,
-  EPHY_PROFILE_DIR_WEB_APP
+  EPHY_PROFILE_DIR_WEB_APP,
+  EPHY_PROFILE_DIR_TEST
 } EphyProfileDirType;
 
 static GHashTable *files;
 static GHashTable *mime_table;
 
 static gboolean keep_directory;
-static char *dot_dir;
+static char *profile_dir_global;
+static char *cache_dir;
+static char *config_dir;
 static char *tmp_dir;
-static GList *del_on_exit;
-static EphyProfileDirType dot_dir_type;
+static EphyProfileDirType profile_dir_type;
 
 GQuark ephy_file_helpers_error_quark;
 
@@ -124,6 +124,8 @@ ephy_file_download_dir (void)
  * Returns a proper downloads destination by checking the
  * EPHY_PREFS_STATE_DOWNLOAD_DIR GSettings key and following this logic:
  *
+ *  - Under flatpak, always use the XDG downloads directory
+ *
  *  - An absolute path: considered user-set, use this value directly.
  *
  *  - "Desktop" keyword in GSettings: the directory returned by
@@ -137,19 +139,20 @@ ephy_file_download_dir (void)
 char *
 ephy_file_get_downloads_dir (void)
 {
-  char *download_dir;
+  g_autofree char *download_dir = g_settings_get_string (EPHY_SETTINGS_STATE,
+                                                         EPHY_PREFS_STATE_DOWNLOAD_DIR);
 
-  download_dir = g_settings_get_string (EPHY_SETTINGS_STATE,
-                                        EPHY_PREFS_STATE_DOWNLOAD_DIR);
+  if (ephy_is_running_inside_flatpak ())
+    return ephy_file_download_dir ();
 
-  if (!g_strcmp0 (download_dir, "Downloads") ||
-      !g_path_is_absolute (download_dir) ||
-      ephy_is_running_inside_flatpak ())
-    download_dir = ephy_file_download_dir ();
-  else if (!g_strcmp0 (download_dir, "Desktop"))
-    download_dir = ephy_file_desktop_dir ();
+  if (g_strcmp0 (download_dir, "Desktop") == 0)
+    return ephy_file_desktop_dir ();
 
-  return download_dir;
+  if (g_strcmp0 (download_dir, "Downloads") == 0 ||
+      !g_path_is_absolute (download_dir))
+    return ephy_file_download_dir ();
+
+  return g_steal_pointer (&download_dir);
 }
 
 /**
@@ -216,7 +219,21 @@ ephy_file_tmp_filename (const char *base,
 }
 
 /**
- * ephy_dot_dir:
+ * ephy_profile_dir:
+ *
+ * Gets Epiphany's configuration directory, usually .local/share/epiphany
+ * under user's homedir.
+ *
+ * Returns: the full path to Epiphany's configuration directory
+ **/
+const char *
+ephy_profile_dir (void)
+{
+  return profile_dir_global;
+}
+
+/**
+ * ephy_config_dir:
  *
  * Gets Epiphany's configuration directory, usually .config/epiphany
  * under user's homedir.
@@ -224,49 +241,96 @@ ephy_file_tmp_filename (const char *base,
  * Returns: the full path to Epiphany's configuration directory
  **/
 const char *
-ephy_dot_dir (void)
+ephy_config_dir (void)
 {
-  return dot_dir;
+  return config_dir;
 }
 
 /**
- * ephy_dot_dir_is_default:
+ * ephy_cache_dir:
  *
- * Returns whether the dot directory in use is the default one, found in
- * ~/.config
+ * Gets Epiphany's cache directory, usually .cache/epiphany
+ * under user's homedir.
  *
- * Returns: %TRUE if it is the default dot dir, %FALSE for others
+ * Returns: the full path to Epiphany's cache directory
+ **/
+const char *
+ephy_cache_dir (void)
+{
+  return cache_dir;
+}
+
+
+/**
+ * ephy_profile_dir_is_default:
+ *
+ * Returns whether the profile directory in use is the default one, found in
+ * ~/.local/share
+ *
+ * Returns: %TRUE if it is the default profile dir, %FALSE for others
  **/
 gboolean
-ephy_dot_dir_is_default (void)
+ephy_profile_dir_is_default (void)
 {
-  return dot_dir_type == EPHY_PROFILE_DIR_DEFAULT;
+  return profile_dir_type == EPHY_PROFILE_DIR_DEFAULT || profile_dir_type == EPHY_PROFILE_DIR_TEST;
 }
 
 /**
- * ephy_dot_dir_is_web_application:
+ * ephy_profile_dir_is_web_application:
  *
- * Returns whether the dot directory in use is a web appplication one.
+ * Returns whether the profile directory in use is a web appplication one.
  *
- * Returns: %TRUE if it is a web application dot dir, %FALSE for others
+ * Returns: %TRUE if it is a web application profile dir, %FALSE for others
  */
 gboolean
-ephy_dot_dir_is_web_application (void)
+ephy_profile_dir_is_web_application (void)
 {
-  return dot_dir_type == EPHY_PROFILE_DIR_WEB_APP;
+  return profile_dir_type == EPHY_PROFILE_DIR_WEB_APP;
 }
 
 /**
- * ephy_default_dot_dir:
+ * ephy_default_profile_dir:
  *
- * Get the path to the default dot directory found in ~/.config
+ * Get the path to the default profile directory found in ~/.local/share
  *
  * Returns: a new allocated string, free with g_free() when done.
  */
 char *
-ephy_default_dot_dir (void)
+ephy_default_profile_dir (void)
 {
-  return g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
+  return profile_dir_type == EPHY_PROFILE_DIR_TEST ?
+    g_strdup (ephy_profile_dir ()) :
+    g_build_filename (g_get_user_data_dir (), "epiphany", NULL);
+}
+
+/**
+ * ephy_default_cache_dir:
+ *
+ * Get the path to the default cache directory found in ~/.cache
+ *
+ * Returns: a new allocated string, free with g_free() when done.
+ */
+char *
+ephy_default_cache_dir (void)
+{
+  return profile_dir_type == EPHY_PROFILE_DIR_TEST ?
+    g_build_filename (ephy_profile_dir (), "cache", NULL) :
+    g_build_filename (g_get_user_cache_dir (), "epiphany", NULL);
+}
+
+/**
+ * ephy_default_config_dir:
+ *
+ * Get the path to the default config directory found in ~/.config
+ *
+ * Returns: a new allocated string, free with g_free() when done.
+ */
+char *
+ephy_default_config_dir (void)
+{
+  return profile_dir_type == EPHY_PROFILE_DIR_TEST ?
+    g_build_filename (ephy_profile_dir (), "config", NULL) :
+    g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
 }
 
 /**
@@ -297,24 +361,28 @@ ephy_file_helpers_init (const char          *profile_dir,
                                  (GDestroyNotify)g_free);
 
   keep_directory = flags & EPHY_FILE_HELPERS_KEEP_DIR;
-  private_profile = flags & EPHY_FILE_HELPERS_PRIVATE_PROFILE;
+  private_profile = (flags & EPHY_FILE_HELPERS_PRIVATE_PROFILE || flags & EPHY_FILE_HELPERS_TESTING_MODE);
   steal_data_from_profile = flags & EPHY_FILE_HELPERS_STEAL_DATA;
 
   if (profile_dir != NULL && !steal_data_from_profile) {
-    char *basename;
-
     if (g_path_is_absolute (profile_dir)) {
-      dot_dir = g_strdup (profile_dir);
+      profile_dir_global = g_strdup (profile_dir);
     } else {
       GFile *file = g_file_new_for_path (profile_dir);
-      dot_dir = g_file_get_path (file);
+      profile_dir_global = g_file_get_path (file);
       g_object_unref (file);
     }
 
-    basename = g_path_get_basename (dot_dir);
-    if (g_str_has_prefix (basename, EPHY_WEB_APP_PREFIX))
-      dot_dir_type = EPHY_PROFILE_DIR_WEB_APP;
-    g_free (basename);
+    g_autofree char *app_file = g_build_filename (profile_dir, ".app", NULL);
+    if (g_file_test (app_file, G_FILE_TEST_EXISTS)) {
+      const char *app_name = ephy_web_application_get_program_name_from_profile_directory (profile_dir_global);
+      cache_dir = g_build_filename (g_get_user_cache_dir (), app_name, NULL);
+      config_dir = g_build_filename (g_get_user_config_dir (), app_name, NULL);
+      profile_dir_type = EPHY_PROFILE_DIR_WEB_APP;
+    } else {
+      cache_dir = g_build_filename (profile_dir_global, "cache", NULL);
+      config_dir = g_build_filename (profile_dir_global, "config", NULL);
+    }
   } else if (private_profile) {
     if (ephy_file_tmp_dir () == NULL) {
       g_set_error (error,
@@ -325,18 +393,31 @@ ephy_file_helpers_init (const char          *profile_dir,
       return FALSE;
     }
 
-    dot_dir = g_build_filename (ephy_file_tmp_dir (),
-                                "epiphany",
-                                NULL);
+    profile_dir_global = g_build_filename (ephy_file_tmp_dir (),
+                                           "epiphany",
+                                           NULL);
+    cache_dir = g_build_filename (profile_dir_global, "cache", NULL);
+    config_dir = g_build_filename (profile_dir_global, "config", NULL);
+    if (flags & EPHY_FILE_HELPERS_TESTING_MODE)
+      profile_dir_type = EPHY_PROFILE_DIR_TEST;
   }
 
-  if (dot_dir == NULL) {
-    dot_dir = ephy_default_dot_dir ();
-    dot_dir_type = EPHY_PROFILE_DIR_DEFAULT;
+  if (profile_dir_global == NULL) {
+    profile_dir_type = EPHY_PROFILE_DIR_DEFAULT;
+    profile_dir_global = ephy_default_profile_dir ();
   }
 
-  if (flags & EPHY_FILE_HELPERS_ENSURE_EXISTS)
-    ret = ephy_ensure_dir_exists (ephy_dot_dir (), error);
+  if (cache_dir == NULL)
+    cache_dir = ephy_default_cache_dir ();
+
+  if (config_dir == NULL)
+    config_dir = ephy_default_config_dir ();
+
+  if (flags & EPHY_FILE_HELPERS_ENSURE_EXISTS) {
+    ret = ephy_ensure_dir_exists (ephy_profile_dir (), error);
+    ephy_ensure_dir_exists (ephy_cache_dir (), NULL);
+    ephy_ensure_dir_exists (ephy_config_dir (), NULL);
+  }
 
   if (steal_data_from_profile && profile_dir) {
     guint i;
@@ -353,7 +434,7 @@ ephy_file_helpers_init (const char          *profile_dir,
       source = g_file_new_for_path (filename);
       g_free (filename);
 
-      filename = g_build_filename (dot_dir,
+      filename = g_build_filename (profile_dir_global,
                                    files_to_copy[i],
                                    NULL);
       destination = g_file_new_for_path (filename);
@@ -375,14 +456,6 @@ ephy_file_helpers_init (const char          *profile_dir,
   return ret;
 }
 
-static void
-delete_files (GList *l)
-{
-  for (; l != NULL; l = l->next) {
-    unlink (l->data);
-  }
-}
-
 /**
  * ephy_file_helpers_shutdown:
  *
@@ -393,20 +466,15 @@ ephy_file_helpers_shutdown (void)
 {
   g_hash_table_destroy (files);
 
-  del_on_exit = g_list_reverse (del_on_exit);
-  delete_files (del_on_exit);
-  g_list_foreach (del_on_exit, (GFunc)g_free, NULL);
-  g_list_free (del_on_exit);
-  del_on_exit = NULL;
-
   if (mime_table != NULL) {
     LOG ("Destroying mime type hashtable");
     g_hash_table_destroy (mime_table);
     mime_table = NULL;
   }
 
-  g_free (dot_dir);
-  dot_dir = NULL;
+  g_clear_pointer (&profile_dir_global, g_free);
+  g_clear_pointer (&cache_dir, g_free);
+  g_clear_pointer (&config_dir, g_free);
 
   if (tmp_dir != NULL) {
     if (!keep_directory) {
@@ -449,7 +517,7 @@ ephy_ensure_dir_exists (const char *dir,
 
   if (!g_file_test (dir, G_FILE_TEST_EXISTS)) {
     if (g_mkdir_with_parents (dir, 488) == 0) {
-      if (dir == ephy_dot_dir ()) {
+      if (dir == ephy_profile_dir ()) {
         /* We need to set the .migrated file to the
          * current profile migration version,
          * otherwise the next time the browser runs
@@ -470,187 +538,33 @@ ephy_ensure_dir_exists (const char *dir,
   return TRUE;
 }
 
-static void
-ephy_find_file_recursive (const char *path,
-                          const char *fname,
-                          GSList    **list,
-                          gint        depth,
-                          gint        maxdepth)
+static gboolean
+launch_application (GAppInfo  *app,
+                    GList     *files,
+                    guint32    user_time)
 {
-  GDir *dir;
-  const gchar *file;
-
-  dir = g_dir_open (path, 0, NULL);
-  if (dir != NULL) {
-    while ((file = g_dir_read_name (dir))) {
-      if (depth < maxdepth) {
-        char *new_path = g_build_filename (path, file, NULL);
-        ephy_find_file_recursive (new_path, fname, list,
-                                  depth + 1, maxdepth);
-        g_free (new_path);
-      }
-      if (strcmp (file, fname) == 0) {
-        char *new_path = g_build_filename (path, file, NULL);
-        *list = g_slist_prepend (*list, new_path);
-      }
-    }
-
-    g_dir_close (dir);
-  }
-}
-
-/**
- * ephy_file_find:
- * @path: path to search for @fname
- * @fname: filename to search for
- * @maxdepth: maximum directory depth when searching @path
- *
- * Searchs for @fname in @path with a maximum depth of @maxdepth.
- *
- * Returns: a GSList of matches
- **/
-GSList *
-ephy_file_find (const char *path,
-                const char *fname,
-                gint        maxdepth)
-{
-  GSList *ret = NULL;
-  ephy_find_file_recursive (path, fname, &ret, 0, maxdepth);
-  return ret;
-}
-
-/**
- * ephy_file_delete_on_exit:
- * @file: a #GFile
- *
- * Schedules @file to be deleted when Epiphany exits. This function currently
- * does nothing.
- **/
-void
-ephy_file_delete_on_exit (GFile *file)
-{
-  /* does nothing now */
-}
-
-static void
-load_mime_from_xml (void)
-{
-  xmlTextReaderPtr reader;
-  const xmlChar *xml_file;
-  int ret;
-  GBytes *bytes;
-  EphyMimePermission permission = EPHY_MIME_PERMISSION_UNKNOWN;
-
-  g_assert (mime_table == NULL);
-
-  mime_table = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                      xmlFree, NULL);
-
-  bytes = g_resources_lookup_data (EPHY_MIME_TYPE_PERMISSIONS_URI, 0, NULL);
-  xml_file = (xmlChar *)g_bytes_get_data (bytes, NULL);
-  reader = xmlReaderForDoc (xml_file, EPHY_MIME_TYPE_PERMISSIONS_URI, NULL, 0);
-  if (reader == NULL) {
-    g_warning ("Could not load MIME types permissions file!\n");
-    return;
-  }
-
-  ret = xmlTextReaderRead (reader);
-  while (ret == 1) {
-    const xmlChar *tag;
-    xmlReaderTypes type;
-
-    tag = xmlTextReaderConstName (reader);
-    type = xmlTextReaderNodeType (reader);
-
-    if (xmlStrEqual (tag, (const xmlChar *)"safe") && type == XML_READER_TYPE_ELEMENT) {
-      permission = EPHY_MIME_PERMISSION_SAFE;
-    } else if (xmlStrEqual (tag, (const xmlChar *)"unsafe") && type == XML_READER_TYPE_ELEMENT) {
-      permission = EPHY_MIME_PERMISSION_UNSAFE;
-    } else if (xmlStrEqual (tag, (const xmlChar *)"mime-type")) {
-      xmlChar *t;
-
-      t = xmlTextReaderGetAttribute (reader, (const xmlChar *)"type");
-      g_hash_table_insert (mime_table, t,
-                           GINT_TO_POINTER (permission));
-    }
-
-    ret = xmlTextReaderRead (reader);
-  }
-
-  xmlFreeTextReader (reader);
-  g_bytes_unref (bytes);
-}
-
-/**
- * ephy_file_check_mime:
- * @mime_type: a mime type
- *
- * Checks @mime_type against our safe/unsafe database of types, returns an
- * #EphyMimePermission.
- *
- * Returns: an #EphyMimePermission
- **/
-EphyMimePermission
-ephy_file_check_mime (const char *mime_type)
-{
-  EphyMimePermission permission;
-  gpointer tmp;
-
-  g_assert (mime_type != NULL);
-
-  if (mime_table == NULL) {
-    load_mime_from_xml ();
-  }
-
-  tmp = g_hash_table_lookup (mime_table, mime_type);
-  if (tmp == NULL) {
-    permission = EPHY_MIME_PERMISSION_UNKNOWN;
-  } else {
-    permission = GPOINTER_TO_INT (tmp);
-  }
-
-  return permission;
-}
-
-/**
- * ephy_file_launch_application:
- * @app: the application to launch
- * @list: files to pass to @app
- * @user_time: user time to prevent focus stealing
- * @widget: a relevant widget from where to get the #GdkScreen and #GdkDisplay
- *
- * Launches @app to open @files. If @widget is set the screen and display from
- * it will be used to launch the application, otherwise the defaults will be
- * used.
- *
- * Returns: %TRUE if g_app_info_launch() succeeded
- **/
-gboolean
-ephy_file_launch_application (GAppInfo  *app,
-                              GList     *list,
-                              guint32    user_time,
-                              GtkWidget *widget)
-{
-  GdkAppLaunchContext *context;
+  g_autoptr(GdkAppLaunchContext) context = NULL;
+  g_autoptr(GError) error = NULL;
   GdkDisplay *display;
   GdkScreen *screen;
   gboolean res;
 
-  if (widget) {
-    display = gtk_widget_get_display (widget);
-    screen = gtk_widget_get_screen (widget);
-  } else {
-    display = gdk_display_get_default ();
-    screen = gdk_screen_get_default ();
-  }
+  /* This is impossible to implement inside flatpak. Higher layers must
+   * ensure we don't get here.
+   */
+  g_assert (!ephy_is_running_inside_flatpak ());
+
+  display = gdk_display_get_default ();
+  screen = gdk_screen_get_default ();
 
   context = gdk_display_get_app_launch_context (display);
   gdk_app_launch_context_set_screen (context, screen);
   gdk_app_launch_context_set_timestamp (context, user_time);
 
-  res = g_app_info_launch (app, list,
-                           G_APP_LAUNCH_CONTEXT (context), NULL);
-  g_object_unref (context);
+  res = g_app_info_launch (app, files,
+                           G_APP_LAUNCH_CONTEXT (context), &error);
+  if (!res)
+    g_warning ("Failed to launch %s: %s", g_app_info_get_display_name (app), error->message);
 
   return res;
 }
@@ -658,78 +572,54 @@ ephy_file_launch_application (GAppInfo  *app,
 /**
  * ephy_file_launch_desktop_file:
  * @filename: the path to the .desktop file
- * @parameter: path to an optional parameter file to pass to the application
- * @user_time: user time to prevent focus stealing
- * @widget: an optional widget for ephy_file_launch_application()
+ * @tag: used to guard against improper usage
  *
- * Calls ephy_file_launch_application() for the application described by the
- * .desktop file @filename. Can pass @parameter as optional file arguments.
+ * Launches the application described by the desktop file @filename.
  *
  * Returns: %TRUE if the application launch was successful
  **/
 gboolean
-ephy_file_launch_desktop_file (const char *filename,
-                               const char *parameter,
-                               guint32     user_time,
-                               GtkWidget  *widget)
+ephy_file_launch_desktop_file (const char                   *filename,
+                               guint32                       user_time,
+                               EphyFileHelpersNotFlatpakTag  tag)
 {
-  GDesktopAppInfo *app;
-  GFile *file = NULL;
-  GList *list = NULL;
-  gboolean ret;
+  g_autoptr(GDesktopAppInfo) app = NULL;
+
+  /* This is impossible to implement inside flatpak. Higher layers must
+   * ensure we don't get here.
+   */
+  g_assert (tag == EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
+  g_assert (!ephy_is_running_inside_flatpak ());
 
   app = g_desktop_app_info_new (filename);
-  if (parameter) {
-    file = g_file_new_for_path (parameter);
-    list = g_list_append (list, file);
-  }
 
-  ret = ephy_file_launch_application (G_APP_INFO (app), list, user_time, widget);
-  g_list_free (list);
-  if (file)
-    g_object_unref (file);
-  return ret;
+  return launch_application (G_APP_INFO (app), NULL, user_time);
 }
 
-GAppInfo *
-ephy_file_launcher_get_app_info_for_file (GFile      *file,
-                                          const char *mime_type)
+static gboolean
+launch_via_uri_handler (GFile *file)
 {
-  GAppInfo *app = NULL;
+  const char *uri;
+  GdkDisplay *display;
+  GdkAppLaunchContext *context;
+  g_autoptr(GError) error = NULL;
 
-  g_assert (file || mime_type);
+  display = gdk_display_get_default ();
+  context = gdk_display_get_app_launch_context (display);
 
-  if (mime_type != NULL) {
-    app = g_app_info_get_default_for_type (mime_type,
-                                           FALSE);
-  } else {
-    GFileInfo *file_info;
-    char *type;
+  uri = g_file_get_uri (file);
 
-    /* Sniff mime type and check if it's safe to open */
-    file_info = g_file_query_info (file,
-                                   G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-                                   0, NULL, NULL);
-    if (file_info == NULL) {
-      return FALSE;
-    }
-    type = g_strdup (g_file_info_get_content_type (file_info));
-
-    g_object_unref (file_info);
-
-    if (type != NULL && type[0] != '\0' &&
-        ephy_file_check_mime (type) == EPHY_MIME_PERMISSION_SAFE) {
-      app = g_app_info_get_default_for_type (type, FALSE);
-    }
-    g_free (type);
+  g_app_info_launch_default_for_uri (uri, G_APP_LAUNCH_CONTEXT (context), &error);
+  if (error) {
+    g_warning ("Failed to launch handler for URI %s: %s", uri, error->message);
+    return FALSE;
   }
 
-  return app;
+  return TRUE;
 }
 
 /**
  * ephy_file_launch_handler:
- * @mime_type: the mime type of @file or %NULL
  * @file: a #GFile to pass as argument
  * @user_time: user time to prevent focus stealing
  *
@@ -739,75 +629,99 @@ ephy_file_launcher_get_app_info_for_file (GFile      *file,
  * Returns: %TRUE on success
  **/
 gboolean
-ephy_file_launch_handler (const char *mime_type,
-                          GFile      *file,
-                          guint32     user_time)
+ephy_file_launch_handler (GFile   *file,
+                          guint32  user_time)
 {
   GAppInfo *app = NULL;
   gboolean ret = FALSE;
+  g_autoptr(GList) list = NULL;
+  g_autoptr(GError) error = NULL;
 
   g_assert (file != NULL);
 
-  app = ephy_file_launcher_get_app_info_for_file (file, mime_type);
+  /* Launch via URI handler only under flatpak, because this way loses
+   * focus stealing prevention. There's no other way to open a file
+   * under flatpak, and focus stealing prevention becomes the
+   * responsibility of the portal in this case anyway. */
+  if (ephy_is_running_inside_flatpak ())
+    return launch_via_uri_handler (file);
 
-  if (app != NULL) {
-    GList *list = NULL;
-
-    list = g_list_append (list, file);
-    ret = ephy_file_launch_application (app, list, user_time, NULL);
-    g_list_free (list);
+  app = g_file_query_default_handler (file, NULL, &error);
+  if (!app) {
+    g_autofree char *path = g_file_get_path (file);
+    g_warning ("No available application to open %s: %s", path, error->message);
+    return FALSE;
   }
 
+  list = g_list_append (list, file);
+  ret = launch_application (app, list, user_time);
+
   return ret;
 }
 
-gboolean
-ephy_file_open_uri_in_default_browser (const char *uri,
-                                       guint32     timestamp,
-                                       GdkScreen  *screen)
+static gboolean
+open_in_default_handler (const char                   *uri,
+                         const char                   *mime_type,
+                         guint32                       timestamp,
+                         GdkScreen                    *screen,
+                         EphyFileHelpersNotFlatpakTag  tag)
 {
-  GdkAppLaunchContext *context;
-  GAppInfo *appinfo;
+  g_autoptr(GdkAppLaunchContext) context = NULL;
+  g_autoptr(GAppInfo) appinfo = NULL;
+  g_autoptr(GError) error = NULL;
   GList uris;
-  gboolean retval = TRUE;
-  GError *error = NULL;
+
+  /* This is impossible to implement inside flatpak. Higher layers must
+   * ensure we don't get here.
+   */
+  g_assert (tag == EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
+  g_assert (!ephy_is_running_inside_flatpak ());
 
   context = gdk_display_get_app_launch_context (screen ? gdk_screen_get_display (screen) : gdk_display_get_default ());
   gdk_app_launch_context_set_screen (context, screen);
   gdk_app_launch_context_set_timestamp (context, timestamp);
 
-  appinfo = g_app_info_get_default_for_type ("x-scheme-handler/http", TRUE);
+  appinfo = g_app_info_get_default_for_type (mime_type, TRUE);
   uris.data = (gpointer)uri;
   uris.next = uris.prev = NULL;
 
   if (!g_app_info_launch_uris (appinfo, &uris, G_APP_LAUNCH_CONTEXT (context), &error)) {
     g_warning ("Failed to launch %s: %s", uri, error->message);
-    g_error_free (error);
-    retval = FALSE;
+    return FALSE;
   }
 
-  g_object_unref (context);
-  g_object_unref (appinfo);
+  return TRUE;
+}
 
-  return retval;
+gboolean
+ephy_file_open_uri_in_default_browser (const char                   *uri,
+                                       guint32                       user_time,
+                                       GdkScreen                    *screen,
+                                       EphyFileHelpersNotFlatpakTag  tag)
+{
+  return open_in_default_handler (uri, "x-scheme-handler/http", user_time, screen, tag);
 }
 
 /**
  * ephy_file_browse_to:
  * @file: a #GFile
  * @user_time: user_time to prevent focus stealing
+ * @tag: used to guard against improper usage
  *
- * Launches the default application for browsing directories, with @file's
- * parent directory as its target. Passes @user_time to
- * ephy_file_launch_handler() to prevent focus stealing.
+ * Launches the default application for browsing directories to point to
+ * @file. E.g. nautilus will jump to @file within its directory and
+ * select it.
  *
  * Returns: %TRUE if the launch succeeded
  **/
 gboolean
-ephy_file_browse_to (GFile  *file,
-                     guint32 user_time)
+ephy_file_browse_to (GFile                        *file,
+                     guint32                       user_time,
+                     EphyFileHelpersNotFlatpakTag  tag)
 {
-  return ephy_file_launch_handler ("inode/directory", file, user_time);
+  g_autofree char *uri = g_file_get_uri (file);
+
+  return open_in_default_handler (uri, "inode/directory", user_time, NULL, tag);
 }
 
 /**
@@ -872,118 +786,6 @@ ephy_file_delete_dir_recursively (const char *directory, GError **error)
   return !failed;
 }
 
-/**
- * ephy_file_delete_uri
- * @uri: URI of the file to be deleted
- *
- * Remove the given URI.
- */
-void
-ephy_file_delete_uri (const char *uri)
-{
-  GFile *file;
-  gboolean ret;
-
-  g_assert (uri);
-
-  file = g_file_new_for_uri (uri);
-
-  ret = g_file_delete (file, NULL, NULL);
-
-  if (ret == TRUE) {
-    LOG ("Deleted file at URI '%s'", uri);
-  } else {
-    LOG ("Couldn't file at URI '%s'", uri);
-  }
-  g_object_unref (file);
-}
-
-/**
- * ephy_file_move_uri
- * @source_uri: URI of the file to be moved
- * @dest_uri: URI of the destination
- *
- * Move from source_uri to dest_uri, overwriting if necessary.
- *
- * Returns: %TRUE on successful move, %FALSE otherwise.
- */
-gboolean
-ephy_file_move_uri (const char *source_uri, const char *dest_uri)
-{
-  GFile *src;
-  GFile *dest;
-  gboolean ret;
-
-  g_assert (source_uri && dest_uri);
-
-  src = g_file_new_for_uri (source_uri);
-  dest = g_file_new_for_uri (dest_uri);
-
-  ret = g_file_move (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_ALL_METADATA,
-                     NULL, NULL, NULL, NULL);
-
-  if (ret == TRUE) {
-    LOG ("Moved file '%s' to '%s'", source_uri, dest_uri);
-  } else {
-    LOG ("Couldn't move file '%s' to '%s'", source_uri, dest_uri);
-  }
-  g_object_unref (src);
-  g_object_unref (dest);
-  return ret;
-}
-
-/**
- * ephy_file_create_data_uri_for_filename:
- * @filename: the filename of a local path
- * @mime_type: the MIME type of the filename, or %NULL
- *
- * Create a data uri using the contents of @filename.
- * If @mime_type is %NULL, the %G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE
- * attribute of @filename will be used.
- *
- * Returns: a new allocated string containg the data uri, or %NULL if the
- *   data uri could not be created
- */
-char *
-ephy_file_create_data_uri_for_filename (const char *filename,
-                                        const char *mime_type)
-{
-  gchar *data;
-  gsize data_length;
-  gchar *base64;
-  gchar *uri = NULL;
-  GFileInfo *file_info = NULL;
-
-  g_assert (filename != NULL);
-
-  if (!g_file_get_contents (filename, &data, &data_length, NULL))
-    return NULL;
-
-  base64 = g_base64_encode ((const guchar *)data, data_length);
-  g_free (data);
-
-  if (!mime_type) {
-    GFile *file;
-
-    file = g_file_new_for_path (filename);
-    file_info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-                                   G_FILE_QUERY_INFO_NONE, NULL, NULL);
-    if (file_info)
-      mime_type = g_file_info_get_content_type (file_info);
-
-    g_object_unref (file);
-  }
-
-  if (mime_type)
-    uri = g_strdup_printf ("data:%s;charset=utf8;base64,%s", mime_type, base64);
-  g_free (base64);
-
-  if (file_info)
-    g_object_unref (file_info);
-
-  return uri;
-}
-
 /**
  * ephy_sanitize_filename:
  * @filename: a filename
@@ -1010,7 +812,7 @@ ephy_open_default_instance_window (void)
 {
   GError *error = NULL;
 
-  g_spawn_command_line_async ("epiphany", &error);
+  g_spawn_command_line_async ("epiphany --new-window", &error);
 
   if (error) {
     g_warning ("Couldn't open default instance: %s", error->message);
@@ -1024,7 +826,7 @@ ephy_open_incognito_window (const char *uri)
   char *command;
   GError *error = NULL;
 
-  command = g_strdup_printf ("epiphany --incognito-mode --profile %s ", ephy_dot_dir ());
+  command = g_strdup_printf ("epiphany --incognito-mode --profile %s ", ephy_profile_dir ());
 
   if (uri) {
     char *str = g_strconcat (command, uri, NULL);
@@ -1041,33 +843,3 @@ ephy_open_incognito_window (const char *uri)
 
   g_free (command);
 }
-
-gboolean
-ephy_file_launch_via_uri_handler (const char *uri)
-{
-  GdkDisplay *display;
-  GdkAppLaunchContext *context;
-  GError *error = NULL;
-
-  display = gdk_display_get_default ();
-  context = gdk_display_get_app_launch_context (display);
-
-  g_app_info_launch_default_for_uri (uri, G_APP_LAUNCH_CONTEXT (context), &error);
-
-  if (error != NULL) {
-    g_warning ("Failed to launch handler for URI %s: %s", uri, error->message);
-    g_error_free (error);
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-gboolean
-ephy_file_launch_file_via_uri_handler (GFile *file)
-{
-  const char *uri;
-
-  uri = g_file_get_uri (file);
-  return ephy_file_launch_via_uri_handler (uri);
-}
